//-------+---------+---------+---------+---------+---------+---------+-----
// view.c - view window function
//-------+---------+---------+---------+---------+---------+---------+-----

///////////////////////////////////////////////////////////////////////////
// Copyright (C) GENERAL SCIENCE (M.Kasahara) since 1999.01
// Programmed by GENERAL SCIENCE (M.Kasahara)
// Ver00.00 1999.01 - wingman Ver00.00
// Ver01.00 1999.06 - wingman Ver00.50
// Ver01.01 2000.01 - wingman Ver01.00
// Ver01.10 2000.08 - wingman Ver01.01
///////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <WGM/mwingman.h>

static wgmImage *g_img = NULL;
static int g_nVPos = 0, g_nHPos = 0;

//-------+---------+---------+---------+---------+---------+---------+-----
// Define Prototype :
//-------+---------+---------+---------+---------+---------+---------+-----

static int WndProc_View (Display *pDspy, wgmWindow *win, XEvent *event);
static int Draw_View (Display *pDspy, wgmWindow *win);

//-------+---------+---------+---------+---------+---------+---------+-----
// Main function :
//-------+---------+---------+---------+---------+---------+---------+-----
// Function List >>> CreateWindow_View :
//               >>> WndProc_View :
//               >>> Draw_View :

///////////////////////////////////////////////////////////////////////////
//>>> CreateWindow_View :
///////////////////////////////////////////////////////////////////////////
wgmWindow *CreateWindow_View (Display *pDspy, wgmWindow *winParent,
  XRectangle rect, char *pCaption)
{
  wgmWindow *pWin;

  pWin = wgmCreateWindow (pDspy, winParent, rect, 1, pCaption, WndProc_View);

  if (pWin == NULL)
    return NULL;

  wgmSetRedrawBuffer (pWin, TRUE);
  wgmSetAutoRedraw (pWin, TRUE);

  return pWin;
}

///////////////////////////////////////////////////////////////////////////
//>>> WndProc_View :
///////////////////////////////////////////////////////////////////////////
static int WndProc_View (Display *pDspy, wgmWindow *win, XEvent *event)
{
  int nRet = win->nEntryRetCode; //TRUE;

  switch (event->type){
  case MapNotify:
    if (event->xmap.window != win->win)
      break;
    Draw_View (pDspy, win);
    break;
  case ConfigureNotify:
    if (event->xconfigure.window != win->win)
      break;
    Draw_View (pDspy, win);
    break;
  case DestroyNotify:
    break;
  case Expose:
  case ClientMessage:
  case EnterNotify:
  case LeaveNotify:
  case ButtonPress:
  case ButtonRelease:
  case MotionNotify:
  case wgmConstructionEVENT:
  case wgmDestructionEVENT:
    break;
  case wgmRedrawEVENT:
    Draw_View (pDspy, win);
    break;
  case wgmDefaultEVENT:
    break;
  }

  return nRet;
}

///////////////////////////////////////////////////////////////////////////
//>>> Draw_View :
///////////////////////////////////////////////////////////////////////////
static int Draw_View (Display *pDspy, wgmWindow *win)
{
  int x, y, nVPos, nHPos;

  wgmClearWindow (pDspy, win, TRUE);

  if (g_img != NULL)
  {
    if (win->rect.width < g_img->nWidth)
    {
      if (g_img->nWidth - win->rect.width < g_nHPos)
        nHPos = g_img->nWidth - win->rect.width;
      else
        nHPos = g_nHPos;
      x = 0;
    }else{
      nHPos = 0;
      x = (win->rect.width - g_img->nWidth) / 2;
    }
    if (win->rect.height < g_img->nHeight)
    {
      if (g_img->nHeight - win->rect.height < g_nVPos)
        nVPos = g_img->nHeight - win->rect.height;
      else
        nVPos = g_nVPos;
      y = 0;
    }else{
      nVPos = 0;
      y = (win->rect.height - g_img->nHeight) / 2;
    }
    wgmDrawImage (pDspy, wgmGetDraw (win), x, y, 
      g_img, nHPos, nVPos, g_img->nWidth, g_img->nHeight);
  }
  //wgmSetAutoRedrawArea (win, wgmSetRect (10, 10, 50, 50));

  //wgmRedrawWindow (pDspy, win);

  return TRUE;
}

//-------+---------+---------+---------+---------+---------+---------+-----
// Member function :
//-------+---------+---------+---------+---------+---------+---------+-----
// Function List >>> GetVPos_View :
//               >>> SetVPos_View :
//               >>> GetHPos_View :
//               >>> SetHPos_View :
//               >>> SetImage_View :

///////////////////////////////////////////////////////////////////////////
//>>> GetVPos_View :
///////////////////////////////////////////////////////////////////////////
int GetVPos_View (wgmWindow *win)
{
  return g_nVPos;
}

///////////////////////////////////////////////////////////////////////////
//>>> SetVPos_View :
///////////////////////////////////////////////////////////////////////////
int SetVPos_View (wgmWindow *win, int nVPos)
{
  if (nVPos < 0)
    return FALSE;

  if (g_nVPos != nVPos)
  {
    g_nVPos = nVPos;
    wgmSendEvent (win, wgmRedrawEVENT);
  }

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> GetHPos_View :
///////////////////////////////////////////////////////////////////////////
int GetHPos_View (wgmWindow *win)
{
  return g_nHPos;
}

///////////////////////////////////////////////////////////////////////////
//>>> SetHPos_View :
///////////////////////////////////////////////////////////////////////////
int SetHPos_View (wgmWindow *win, int nHPos)
{
  if (nHPos < 0)
    return FALSE;

  if (g_nHPos != nHPos)
  {
    g_nHPos = nHPos;
    wgmSendEvent (win, wgmRedrawEVENT);
  }

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> SetImage_View :
///////////////////////////////////////////////////////////////////////////
int SetImage_View (wgmWindow *win, wgmImage *img)
{
  if (g_img != img)
  {
    g_img = img;
    wgmSendEvent (win, wgmRedrawEVENT);
  }

  return TRUE;
}

