//-------+---------+---------+---------+---------+---------+---------+-----
// wingview.c - Wingman image viewer for X window system Ver01.10
//-------+---------+---------+---------+---------+---------+---------+-----

///////////////////////////////////////////////////////////////////////////
// Copyright (C) GENERAL SCIENCE (M.Kasahara) since 1999.01
// Programmed by GENERAL SCIENCE (M.Kasahara)
// Ver00.00 1999.01 - wingman Ver00.00
// Ver01.00 1999.06 - wingman Ver00.50
// Ver01.01 2000.01 - wingman Ver01.00
// Ver01.10 2000.08 - wingman Ver01.01
///////////////////////////////////////////////////////////////////////////
// !!! Information !!! - Find ">>>" for find function in this source.
///////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <sys/param.h>
#include <WGM/mwingman.h>
#include <X11/keysym.h>

#include "view.h"
#include "resource.h"

#define SCROLLMIN 32

static char g_cFileName [255];
static wgmImage *g_img;

//-------+---------+---------+---------+---------+---------+---------+-----
// Define Prototype :
//-------+---------+---------+---------+---------+---------+---------+-----

static void arginit (int argc, char *argv []);
static void usage (int nUsage);
static int MainLoop (Display *pDspy);
static int WndProc_Main (Display *pDspy, wgmWindow *win, XEvent *event);
static int Draw_Main (Display *pDspy, wgmWindow *win);
static int TakeInPicture (char *pFile);
static int OnVScroll (wgmWindow *win, int nRetCode, int nPos, void *pParam);
static int OnHScroll (wgmWindow *win, int nRetCode, int nPos, void *pParam);
static int OnKeyEvent (wgmWindow *win, XEvent *event, unsigned int key);
static int OnMenuOpen (wgmWindow *win, int nParam1, int nParam2, void *pParam);
static int OnMenuQuit (wgmWindow *win, int nParam1, int nParam2, void *pParam);
static int OnMenuHelp (wgmWindow *win, int nParam1, int nParam2, void *pParam);
static int OnMenuAbout (wgmWindow *win, int nParam1, int nParam2, void *pParam);

//-------+---------+---------+---------+---------+---------+---------+-----
// main function :
//-------+---------+---------+---------+---------+---------+---------+-----
// Function List >>> main :
//               >>> arginit :
//               >>> usage :
//               >>> MainLoop :
//               >>> WndProc_Main :
//               >>> Draw_Main :
//               >>> TakeInPicture :

///////////////////////////////////////////////////////////////////////////
//>>> main :
///////////////////////////////////////////////////////////////////////////
int main (int argc, char *argv [])
{
  Display *pDspy;

  pDspy = wgmOpenDisplay (argc, argv);

  arginit (argc, argv);
  usage (FALSE);

  MainLoop (pDspy);

  wgmCloseDisplay (pDspy);

  return 0;
}

///////////////////////////////////////////////////////////////////////////
//>>> arginit:
///////////////////////////////////////////////////////////////////////////
static void arginit (int argc, char *argv [])
{
  int i;

  for (i=1; i<argc; i++)
  {
    if (!strcmp (argv [i], "-display") // for wingman standard command
      || !strcmp (argv [i], "-d"))
    {
      if (i < argc - 1)
        i++;
    }else if (!strcmp (argv [i], "-geometry") // for wingman standard command
      || !strcmp (argv [i], "-g")){
      if (i < argc - 1)
        i++;
    }else if (!strcmp (argv [i], "-help")
      || !strcmp (argv [i], "--help")
      || !strcmp (argv [i], "--h")
      || !strcmp (argv [i], "--?")
      || !strcmp (argv [i], "-?")
      || !strcmp (argv [i], "-h")){
      usage (TRUE);
      wgmCloseDisplay (DISPLAY);
      exit (0);
    }else{
      strcpy (g_cFileName, argv [i]);
      return;
    }
  }
}

///////////////////////////////////////////////////////////////////////////
//>>> usage:
///////////////////////////////////////////////////////////////////////////
static void usage (int nUsage)
{
  fprintf (stderr, ">>> " APPNAME " <<<\n");
  fprintf (stderr, "- " APPVERSION " " APPDATE " -\n");
  fprintf (stderr, "- Copyright (C) " APPPRODUCT " -\n");
  fprintf (stderr, "\n");
  fprintf (stderr, " X Window Pixmap(*.xpm) and MS-Windows bitmap(*.bmp)\n");
  fprintf (stderr, " and Jpeg(*.jpg) Image file Viewer for Wingman ...  \n");
  fprintf (stderr, "\n");
  if (nUsage == FALSE)
    return;
  fprintf (stderr, " Usage: %s ImageFile\n\n", APPCOMMAND);
  fprintf (stderr, " ImageFile ... Bitmap file name(*.xpm, *.bmp, *.jpg)\n");
  fprintf (stderr, "\n");
}

///////////////////////////////////////////////////////////////////////////
//>>> MainLoop:
///////////////////////////////////////////////////////////////////////////
static int MainLoop (Display *pDspy)
{
  wgmWindow *winMain, *winView, *winVScroll, *winHScroll, *winToolbar;
  wgmWindow *winMenuBar, *winMenu, *winMenuHelp;
  XRectangle rectMain, rectView, rectVS, rectHS;
  int nBar, nTool, nSC;
  wgmSize size;

  // initialize
  g_img = NULL;
  nBar = wgmGetHeight_MenuBar ();
  nTool = wgmGetHeight_ToolBar ();
  nSC = wgmGetHeight_ScrollBar ();
  rectMain = wgmSetRect (0, 0, 5+322+nSC+5, nBar+nTool+5+242+nSC+5);
  rectView = wgmSetRect (5, nBar+nTool+5, 320, 240);
  rectVS = wgmSetRect (5+320+0, nBar+nTool+5, nSC, 240);
  rectHS = wgmSetRect (5, nBar+nTool+5+240+0, 320, nSC);
  size.nMinWidth = 5+322+nSC+5;
  size.nMinHeight = nBar+nTool+5+242+nSC+5;
  size.nMaxWidth = ScreenWIDTH * 2;
  size.nMaxHeight = ScreenHEIGHT * 2;

  // create window & setup
  winMain = wgmCreateWindow (pDspy, NULL, rectMain, 1, "Wingview", 
    WndProc_Main);
  wgmSetMinMaxSize (winMain, size);
  wgmSetKeyEventCallback (winMain, OnKeyEvent);

  winView = CreateWindow_View (pDspy, winMain, rectView, "ViewWnd"); 

  winVScroll = wgmCreateWindow_VScrollBar (pDspy, winMain, rectVS, "VScroll", 
    0, 0, SCROLLMIN, OnVScroll);

  winHScroll = wgmCreateWindow_HScrollBar (pDspy, winMain, rectHS, "HScroll", 
    0, 0, SCROLLMIN, OnHScroll);

  winMenu = wgmCreateWindow_Menu (pDspy, winMain, "MenuWnd");
  wgmAddItem_Menu (winMenu, "Open (o)", OnMenuOpen, DEFImage(WGM_FOLDER), 
    NULL, WGM_INDEX_LAST);
  wgmAddItem_Menu (winMenu, "Quit (q)", OnMenuQuit, NULL, 
    NULL, WGM_INDEX_LAST);

  winMenuHelp = wgmCreateWindow_Menu (pDspy, winMain, "MenuHelpWnd");
  wgmAddItem_Menu (winMenuHelp, "Help (e)", OnMenuHelp, NULL, 
    NULL, WGM_INDEX_LAST);
  wgmAddItem_Menu (winMenuHelp, "About (a)", OnMenuAbout, NULL, 
    NULL, WGM_INDEX_LAST);

  winMenuBar = wgmCreateWindow_MenuBar (pDspy, winMain, "MenuBar", 
    winMenu, winMenuHelp, "Welcome to bmpview!!!(ä㡼)");

  winToolbar = wgmCreateWindow_ToolBar (pDspy, winMain, "ToolBar");
  wgmAddButton_ToolBar (pDspy, winToolbar, "Open", OnMenuOpen, 
    DEFImage(WGM_FOLDER), WGM_INDEX_LAST);
  wgmAddButton_ToolBar (pDspy, winToolbar, "Exit", OnMenuQuit, 
    NULL, WGM_INDEX_LAST);

  wgmSetContextMenu (winView, winMenu);
  wgmCenteringWnd (pDspy, NULL, winMain);
  wgmSetGeometry (winMain, APPGEOMETRY);

  TakeInPicture (g_cFileName); // for command line file

  // window event loop 
  //  - return WGM_QUIT event on Callback function (menu, button, keyevent, ...)
  wgmMainLoop (pDspy, winMain);

  // destruction
  if (g_img != NULL)
    wgmFreeImage (pDspy, g_img);

  wgmDestroyWindow (pDspy, winMain);

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> WndProc_Main :
///////////////////////////////////////////////////////////////////////////
static int WndProc_Main (Display *pDspy, wgmWindow *win, XEvent *event)
{
  int nRet = win->nEntryRetCode; //TRUE;
  wgmWindow *pWin;
  XRectangle rectMain, rectView, rectVS, rectHS;
  int nBar, nTool, nSC;

  switch (event->type){
  case MapNotify:
    if (event->xmap.window != win->win)
      break;
    Draw_Main (pDspy, win);
    break;
  case ConfigureNotify:
    if (event->xconfigure.window != win->win)
      break;

    nBar = wgmGetHeight_MenuBar ();
    nTool = wgmGetHeight_ToolBar ();
    nSC = wgmGetHeight_ScrollBar ();

    rectView = wgmSetRect (5, nBar+nTool+5, 
      win->rect.width-nSC-10-2, win->rect.height-nBar-nTool-nSC-10-2);
    rectVS = wgmSetRect (win->rect.width-nSC-5, nBar+nTool+5, 
      nSC, win->rect.height-nBar-nTool-10-nSC);
    rectHS = wgmSetRect (5, win->rect.height-nSC-5, 
      win->rect.width-10-nSC, nSC);

    XMoveResizeWindow (pDspy, wgmGetWindow ("ViewWnd")->win, 
      rectView.x, rectView.y, rectView.width, rectView.height);
    XMoveResizeWindow (pDspy, wgmGetWindow ("VScroll")->win, 
      rectVS.x, rectVS.y, rectVS.width, rectVS.height);
    XMoveResizeWindow (pDspy, wgmGetWindow ("HScroll")->win, 
      rectHS.x, rectHS.y, rectHS.width, rectHS.height);

    Draw_Main (pDspy, win);
    break;
  case DestroyNotify:
    break;
  case Expose:
    if (event->xexpose.window != win->win)
      break;
    if (event->xexpose.count != 0)
      break;
    Draw_Main (pDspy, win);
    break;
  case ClientMessage:
  case EnterNotify:
  case LeaveNotify:
  case ButtonPress:
  case ButtonRelease:
  case MotionNotify:
  case wgmConstructionEVENT:
  case wgmDestructionEVENT:
    break;
  case wgmRedrawEVENT:
    Draw_Main (pDspy, win);
    break;
  case wgmDefaultEVENT:
    break;
  }

  return nRet;
}

///////////////////////////////////////////////////////////////////////////
//>>> Draw_Main :
///////////////////////////////////////////////////////////////////////////
static int Draw_Main (Display *pDspy, wgmWindow *win)
{
  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> TakeInPicture :
///////////////////////////////////////////////////////////////////////////
static int TakeInPicture (char *pFile)
{
  int nRet = TRUE, nFormat;
  char cText [255], cInfo [80];
  wgmImage *img;
  FILE *fp;

  if (!strlen (pFile))
    return FALSE;

  if ((fp = fopen (pFile, "r")) == NULL)
  {
    sprintf (cText, 
      "This file(%s) is not exist.\nΥեϳޤ", pFile);
    wgmMessageDialog (DISPLAY, wgmGetWindow ("Wingview"), "Error", cText, 
      WGM_MSGDLG_OK | WGM_MSGDLG_ICON_ERROR);
    return FALSE;
  }
  fclose (fp);

  wgmSetWaitCursor (DISPLAY, wgmGetWindow ("Wingview"));

  nFormat = wgmCheckImageFormat (pFile);
  if ((img = wgmLoadImage (DISPLAY, pFile, COLORMAP, nFormat)) != NULL)
  {
    switch (img->nInfo){
    case WGM_IMAGE_UNKNOWN: strcpy (cInfo, "?"); break;
    case WGM_IMAGE_BMPB1:   strcpy (cInfo, "Bitmap 1bpp"); break;
    case WGM_IMAGE_BMPB4:   strcpy (cInfo, "Bitmap 4bpp"); break;
    case WGM_IMAGE_BMPB8:   strcpy (cInfo, "Bitmap 8bpp"); break;
    case WGM_IMAGE_BMPB24:  strcpy (cInfo, "Bitmap 24bpp"); break;
    case WGM_IMAGE_BMP:     strcpy (cInfo, "Bitmap ?bpp"); break;
    case WGM_IMAGE_XPM:     strcpy (cInfo, "Xpm"); break;
    case WGM_IMAGE_JPG:     strcpy (cInfo, "Jpeg"); break;
    }
    sprintf (cText, "%d x %d %s [%s]", 
      img->nWidth, img->nHeight, cInfo, pFile);
    wgmSetStatus_MenuBar (wgmGetWindow ("MenuBar"), cText);

    wgmSetPos_ScrollBar (wgmGetWindow ("VScroll"), 0);
    wgmSetPos_ScrollBar (wgmGetWindow ("HScroll"), 0);
    wgmSetMax_ScrollBar (wgmGetWindow ("VScroll"), img->nHeight);
    wgmSetMax_ScrollBar (wgmGetWindow ("HScroll"), img->nWidth);

    SetImage_View (wgmGetWindow ("ViewWnd"), img);

    if (g_img != NULL)
      wgmFreeImage (DISPLAY, g_img);
    g_img = img;

  }else{
    sprintf (cText, 
      "This file(%s) is not image file.\nΥեϳޤ", pFile);
    wgmMessageDialog (DISPLAY, wgmGetWindow ("Wingview"), "Error", cText, 
      WGM_MSGDLG_OK | WGM_MSGDLG_ICON_ERROR);
  }

  wgmSetNormalCursor (DISPLAY, wgmGetWindow ("Wingview"));

  return nRet;
}

//-------+---------+---------+---------+---------+---------+---------+-----
// Call back function :
//-------+---------+---------+---------+---------+---------+---------+-----
// Function List >>> OnVScroll :
//               >>> OnHScroll :
//               >>> OnKeyEvent :
//               >>> OnMenuOpen :
//               >>> OnMenuQuit :
//               >>> OnMenuHelp :
//               >>> OnMenuAbout :

///////////////////////////////////////////////////////////////////////////
//>>> OnVScroll :
///////////////////////////////////////////////////////////////////////////
static int OnVScroll (wgmWindow *win, int nRetCode, int nPos, void *pParam)
{
  SetVPos_View (wgmGetWindow ("ViewWnd"), nPos * SCROLLMIN);

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> OnHScroll :
///////////////////////////////////////////////////////////////////////////
static int OnHScroll (wgmWindow *win, int nRetCode, int nPos, void *pParam)
{
  SetHPos_View (wgmGetWindow ("ViewWnd"), nPos * SCROLLMIN);

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> OnKeyEvent :
///////////////////////////////////////////////////////////////////////////
static int OnKeyEvent (wgmWindow *win, XEvent *event, unsigned int key)
{
  wgmWindow *winView, *winVScroll, *winHScroll;
  int nRet = TRUE, nPos;

  if (key == XK_o || key == XK_w)
    OnMenuOpen (wgmGetWindow ("MenuWnd"), 0, 0, NULL);

  if (key == XK_q || key == XK_p)
    nRet = OnMenuQuit (wgmGetWindow ("MenuWnd"), 0, 0, NULL);

  if (key == XK_e || key == XK_i)
    OnMenuHelp (wgmGetWindow ("MenuHelpWnd"), 0, 0, NULL);

  if (key == XK_a || key == XK_semicolon)
    OnMenuAbout (wgmGetWindow ("MenuHelpWnd"), 0, 0, NULL);

  if (key == XK_f || key == XK_j || key == XK_Down)
  {
    if (g_img != NULL)
    {
      winVScroll = wgmGetWindow ("VScroll");
      winView = wgmGetWindow ("ViewWnd");
      nPos = wgmGetPos_ScrollBar (winVScroll) + 1;
      if (nPos*SCROLLMIN <= g_img->nHeight-winView->rect.height+SCROLLMIN-1)
        wgmSetPos_ScrollBar (winVScroll, nPos);
    }
  }
  if (key == XK_d || key == XK_k || key == XK_Up)
  {
    if (g_img != NULL)
    {
      winVScroll = wgmGetWindow ("VScroll");
      winView = wgmGetWindow ("ViewWnd");
      nPos = wgmGetPos_ScrollBar (winVScroll) - 1;
      if (0 <= nPos)
        wgmSetPos_ScrollBar (winVScroll, nPos);
    }
  }
  if (key == XK_l || key == XK_g || key == XK_Right)
  {
    if (g_img != NULL)
    {
      winHScroll = wgmGetWindow ("HScroll");
      winView = wgmGetWindow ("ViewWnd");
      nPos = wgmGetPos_ScrollBar (winHScroll) + 1;
      if (nPos*SCROLLMIN <= g_img->nWidth-winView->rect.width+SCROLLMIN-1)
        wgmSetPos_ScrollBar (winHScroll, nPos);
    }
  }
  if (key == XK_h || key == XK_s || key == XK_Left)
  {
    if (g_img != NULL)
    {
      winHScroll = wgmGetWindow ("HScroll");
      winView = wgmGetWindow ("ViewWnd");
      nPos = wgmGetPos_ScrollBar (winHScroll) - 1;
      if (0 <= nPos)
        wgmSetPos_ScrollBar (winHScroll, nPos);
    }
  }

  return nRet;
}

///////////////////////////////////////////////////////////////////////////
//>>> OnMenuOpen :
///////////////////////////////////////////////////////////////////////////
static int OnMenuOpen (wgmWindow *win, int nParam1, int nParam2, void *pParam)
{
  char cFile [MAXPATHLEN];
  int nRet = FALSE;
  char *cExp [] =
    {".bmp", ".BMP", ".xpm", ".XPM", ".jpg", ".JPG", ".jpeg", ".JPEG", 0};

  cFile [0] = '\0';
  wgmFileDialog (DISPLAY, wgmGetWindow ("Wingview"), cFile, cExp, 
    WGM_FILE_OPEN);

  if (strlen (cFile))
  {
    TakeInPicture (cFile);
    nRet = TRUE;
  }else{
    nRet = FALSE;
  }

  return nRet;
}

///////////////////////////////////////////////////////////////////////////
//>>> OnMenuQuit :
///////////////////////////////////////////////////////////////////////////
static int OnMenuQuit (wgmWindow *win, int nParam1, int nParam2, void *pParam)
{
  if (g_img != NULL)
  {
    wgmFreeImage (DISPLAY, g_img);
    g_img = NULL;
  }

  return WGM_QUIT;
}

///////////////////////////////////////////////////////////////////////////
//>>> OnMenuHelp :
///////////////////////////////////////////////////////////////////////////
static int OnMenuHelp (wgmWindow *win, int nParam1, int nParam2, void *pParam)
{
  wgmTextDialog (DISPLAY, wgmGetWindow ("Wingview"), "Wingview Help", 
    res_cHelp);

  return TRUE;
}

///////////////////////////////////////////////////////////////////////////
//>>> OnMenuAbout :
///////////////////////////////////////////////////////////////////////////
static int OnMenuAbout (wgmWindow *win, int nParam1, int nParam2, void *pParam)
{
  wgmImage *imgLogo, *imgWingview;

  imgLogo = wgmLoadImageFromData (DISPLAY, res_logomono, COLORMAP);
  imgWingview = wgmLoadImageFromData (DISPLAY, res_wingview, COLORMAP);

  wgmAboutDialog (DISPLAY, wgmGetWindow ("Wingview"), "Wingview About", 
    ">>> " APPNAME " <<<\n"
    "- " APPVERSION " " APPDATE " -\n"
    "- Copyright (C) " APPPRODUCT " -\n",
    imgWingview, imgLogo, DEFBaseColor(WGM_BLUE));

  wgmFreeImage (DISPLAY, imgLogo);
  wgmFreeImage (DISPLAY, imgWingview);

  return TRUE;
}

